#include <windows.h>
#include <tchar.h>

#define SYSMEM_CHANGED	0

extern "C" BOOL SetSystemDefaultLCID(LCID Locale);
extern "C" void NotifyWinUserSystem(UINT uEvent);
extern "C" BOOL GetSystemMemoryDivision(LPDWORD lpdwStorePages, LPDWORD lpdwRamPages, LPDWORD lpdwPageSize);
extern "C" DWORD SetSystemMemoryDivision(DWORD dwStorePages);

struct CONTROL_PANEL_SETTINGS
{
	COLORREF SystemColors[C_SYS_COLOR_TYPES];
};

TCHAR g_AppPath[MAX_PATH];
TCHAR g_AppDir[MAX_PATH];
HANDLE g_hFile;
BYTE g_FileBuffer[MAX_PATH];
DWORD g_BufferPointer;


BOOL FlushBufferFile()
{
	DWORD i;
	if(g_hFile)
	{
		if(g_BufferPointer > 0)
		{
			WriteFile(g_hFile, &g_FileBuffer, g_BufferPointer, &i, NULL);
			if(i != g_BufferPointer)
				return FALSE;
		}
	}
	g_BufferPointer = 0;
	g_hFile = NULL;
	return TRUE;
}

BOOL WriteBufferFile(HANDLE hFile, LPCVOID lpBuffer, DWORD nNumberOfBytesToWrite, LPDWORD lpNumberOfBytesWritten, LPOVERLAPPED lpOverlapped)
{
	BYTE* pBuf;
	DWORD i;
	DWORD j;
	pBuf = (BYTE*)lpBuffer;
	i = nNumberOfBytesToWrite;
	if(hFile != g_hFile)
	{
		if(!FlushBufferFile())
			return FALSE;
	}
	g_hFile = hFile;
	while(i > 0)
	{
		j = min(i, MAX_PATH - g_BufferPointer);
		memcpy((BYTE*)&g_FileBuffer + g_BufferPointer, pBuf, j);
		g_BufferPointer += j;
		i -= j;
		pBuf += j;
		if(g_BufferPointer >= MAX_PATH)
		{
			WriteFile(g_hFile, &g_FileBuffer, g_BufferPointer, &j, lpOverlapped);
			if(j != g_BufferPointer)
			{
				*lpNumberOfBytesWritten = 0;
				return FALSE;
			}
			g_BufferPointer = 0;
		}
	}
	*lpNumberOfBytesWritten = nNumberOfBytesToWrite;
	return TRUE;
}


BOOL SaveControlPanelSettings(LPCTSTR FileName)
{
	CONTROL_PANEL_SETTINGS Settings;
	HANDLE hFile;
	DWORD dw;
	HKEY hKey;
	INT i;
	for(i = 0; i < C_SYS_COLOR_TYPES; i++)
		Settings.SystemColors[i] = GetSysColor(i | SYS_COLOR_INDEX_FLAG);
	hFile = CreateFile(FileName, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile == INVALID_HANDLE_VALUE)
		return FALSE;
	WriteBufferFile(hFile, &Settings, sizeof(CONTROL_PANEL_SETTINGS), &dw, NULL);
	if(dw != sizeof(CONTROL_PANEL_SETTINGS))
	{
		CloseHandle(hFile);
		return FALSE;
	}
	if(!FlushBufferFile())
	{
		CloseHandle(hFile);
		return FALSE;
	}
	CloseHandle(hFile);
	return TRUE;
}

BOOL LoadControlPanelSettings(LPCTSTR FileName)
{
	CONTROL_PANEL_SETTINGS Settings;
	HANDLE hFile;
	DWORD dw;
	HKEY hKey;
	INT i;
	INT Elements[C_SYS_COLOR_TYPES];
	hFile = CreateFile(FileName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if(hFile == INVALID_HANDLE_VALUE)
		return FALSE;
	ReadFile(hFile, &Settings, sizeof(CONTROL_PANEL_SETTINGS), &dw, NULL);
	if(dw != sizeof(CONTROL_PANEL_SETTINGS))
	{
		CloseHandle(hFile);
		return FALSE;
	}
	CloseHandle(hFile);
	for(i = 0; i < C_SYS_COLOR_TYPES; i++)
		Elements[i] = i | SYS_COLOR_INDEX_FLAG;
	if(!SetSysColors(C_SYS_COLOR_TYPES, (INT*)&Elements, (COLORREF*)&Settings.SystemColors))
		return FALSE;
	return TRUE;
}

int _tWinMain(
    HINSTANCE hInstance, 
    HINSTANCE hPrevInstance, 
    LPTSTR lpCmdLine, 
    int nCmdShow
){
int save = 0;
if(save) {
	if(!wcscmp(lpCmdLine, TEXT(""))) {
		MessageBox(NULL , TEXT("TMK ThemeSave V1.0\n\ntsave.exe <SaveFileName(FullPath)>\n\n\n2022 (C)TMKSoft") , TEXT("About") , MB_OK | MB_TOPMOST | MB_SETFOREGROUND);
		return 0;
	}
	if(!SaveControlPanelSettings(lpCmdLine))
	MessageBox(NULL , TEXT("Failed to write theme file!") , NULL , MB_OK | MB_ICONSTOP | MB_TOPMOST | MB_SETFOREGROUND);
}else{
	if(!wcscmp(lpCmdLine, TEXT(""))) {
		MessageBox(NULL , TEXT("TMK ThemeLoad V1.0\n\ntload.exe <LoadFileName(FullPath)>\n\n\n2022 (C)TMKSoft") , TEXT("About") , MB_OK | MB_TOPMOST | MB_SETFOREGROUND);
		return 0;
	}
	if(!LoadControlPanelSettings(lpCmdLine))
	MessageBox(NULL , TEXT("Failed to load theme file!") , NULL , MB_OK | MB_ICONSTOP | MB_TOPMOST | MB_SETFOREGROUND);
}
}
